@@ -4,16 +4,56 @@
 //! ID information, to upgrade ID number from 15-digit to 18-digit, to generate
 //! fake ID number, and some related functions for HongKong/Macau/Taiwan ID.
 //!
+//! # Examples
+//!
+//! ```
+//! use idcard::{Identity, fake, Gender};
+//!
+//! let id = Identity::new("632123820927051");
+//!
+//! // Determines whether the ID number is valid.
+//! assert_eq!(id.is_valid(), true);
+//!
+//! // Gets properties
+//! let number = id.number();
+//! let gender = id.gender();
+//! let age = id.age();
+//! let birth_date = id.birth_date();
+//! let region = id.region();
+//! // and so on...
+//!
+//! // Converts the value to a JSON string.
+//! println!("{}", id.to_json_string(true));
+//!
+//! // Upgrades an ID number from 15-digit to 18-digit.
+//! let id18 = idcard::upgrade("310112850409522").unwrap();
+//! assert_eq!(&id18, "310112198504095227");
+//!
+//! // Validates an ID number.
+//! assert_eq!(idcard::validate("230127197908177456"), true);
+//!
+//! // Generates a random fake ID number using the given options.
+//! let opts = fake::FakeOptions::new()
+//!     .region("3301")
+//!     .min_year(1990)
+//!     .max_year(2000)
+//!     .gender(Gender::Female);
+//! match fake::rand_with_opts(&opts) {
+//!     Ok(num) => println!("{}", num),
+//!     Err(e) => println!("{}", e),
+//! }
+//! ```
+//! For more information ,please read the API documentation.
+//!
 
 #[macro_use]
 extern crate lazy_static;
 
-use chrono::prelude::*;
-use chrono::{Datelike, Duration, Local, NaiveDate};
-use rand::{thread_rng, Rng};
+use chrono::{Datelike, Local, NaiveDate};
 use std::collections::HashMap;
 use std::fmt;
 
+pub mod fake;
 pub mod hk;
 pub mod mo;
 pub mod region;
@@ -97,14 +137,14 @@ impl fmt::Display for Error {
 }
 
 /// The type of demographic genders
-#[derive(Debug, PartialEq, PartialOrd)]
+#[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord)]
 pub enum Gender {
     Male,
     Female,
 }
 
 /// An object representation of the Chinese ID.
-#[derive(Debug, PartialEq, PartialOrd)]
+#[derive(Debug, Clone, PartialEq, Eq, PartialOrd, Ord)]
 pub struct Identity {
     number: String,
     valid: bool,
@@ -134,11 +174,11 @@ impl Identity {
     }
 
     /// Returns the ID number.
-    pub fn number(&self) -> String {
-        self.number.clone()
+    pub fn number(&self) -> &str {
+        &self.number
     }
 
-    /// Returns the date of birth(yyyy-mm-dd).
+    /// Returns the formatted date of birth(yyyy-mm-dd).
     pub fn birth_date(&self) -> Option<String> {
         if !self.is_valid() {
             return None;
@@ -178,13 +218,13 @@ impl Identity {
     }
 
     /// Returns the day in the month of the birth.
-    pub fn date(&self) -> Option<i32> {
+    pub fn day(&self) -> Option<i32> {
         if !self.is_valid() {
             return None;
         }
 
-        if let Ok(date) = self.number[12..14].parse::<i32>() {
-            Some(date)
+        if let Ok(day) = self.number[12..14].parse::<i32>() {
+            Some(day)
         } else {
             None
         }
@@ -243,20 +283,20 @@ impl Identity {
     }
 
     /// Returns the province name based on the first 2 digits of the number
-    pub fn province(&self) -> Option<String> {
+    pub fn province(&self) -> Option<&str> {
         if !self.is_valid() {
             return None;
         }
 
         let code = &self.number[0..2];
         match PROVINCE_CODE_NAME.get(code) {
-            Some(name) => Some(name.to_string()),
+            Some(name) => Some(*name),
             None => None,
         }
     }
 
     /// Returns the region name based on the first 6 digits of the number
-    pub fn region(&self) -> Option<String> {
+    pub fn region(&self) -> Option<&str> {
         if !self.is_valid() {
             return None;
         }
@@ -264,8 +304,17 @@ impl Identity {
         region::query(&self.number[0..6])
     }
 
+    /// Returns the region code(the first 6 digits)
+    pub fn region_code(&self) -> Option<&str> {
+        if !self.is_valid() {
+            return None;
+        }
+
+        Some(&self.number[0..6])
+    }
+
     /// Returns the constellation by the date of birth.
-    pub fn constellation(&self) -> Option<String> {
+    pub fn constellation(&self) -> Option<&str> {
         if !self.is_valid() {
             return None;
         }
@@ -274,7 +323,7 @@ impl Identity {
             Some(value) => value,
             None => return None,
         };
-        let day = match self.date() {
+        let day = match self.day() {
             Some(value) => value,
             None => return None,
         };
@@ -297,7 +346,7 @@ impl Identity {
     }
 
     /// Returns the Chinese Zodiac animal by the year of birth.
-    pub fn chinese_zodiac(&self) -> Option<String> {
+    pub fn chinese_zodiac(&self) -> Option<&str> {
         if !self.is_valid() {
             return None;
         }
@@ -324,18 +373,95 @@ impl Identity {
     pub fn len(&self) -> usize {
         self.number.len()
     }
+
+    /// Converts the value to a JSON string.
+    pub fn to_json_string(&self, pretty: bool) -> String {
+        let indent = if pretty { "    " } else { "" };
+        let space = if pretty { " " } else { "" };
+        let props = if self.is_valid() {
+            vec![
+                format!(r#"{}"number":{}{:?}"#, indent, space, self.number()),
+                format!(
+                    r#"{}"gender":{}"{:?}""#,
+                    indent,
+                    space,
+                    self.gender().unwrap()
+                ),
+                format!(
+                    r#"{}"birthDate":{}{:?}"#,
+                    indent,
+                    space,
+                    self.birth_date().unwrap()
+                ),
+                format!(r#"{}"year":{}{:?}"#, indent, space, self.year().unwrap()),
+                format!(r#"{}"month":{}{:?}"#, indent, space, self.month().unwrap()),
+                format!(r#"{}"day":{}{:?}"#, indent, space, self.day().unwrap()),
+                format!(r#"{}"age":{}{:?}"#, indent, space, self.age().unwrap()),
+                format!(
+                    r#"{}"province":{}{:?}"#,
+                    indent,
+                    space,
+                    self.province().unwrap()
+                ),
+                format!(
+                    r#"{}"region":{}{:?}"#,
+                    indent,
+                    space,
+                    self.region().unwrap()
+                ),
+                format!(
+                    r#"{}"regionCode":{}{:?}"#,
+                    indent,
+                    space,
+                    self.region_code().unwrap()
+                ),
+                format!(
+                    r#"{}"chineseEra":{}{:?}"#,
+                    indent,
+                    space,
+                    self.chinese_era().unwrap()
+                ),
+                format!(
+                    r#"{}"chineseZodiac":{}{:?}"#,
+                    indent,
+                    space,
+                    self.chinese_zodiac().unwrap()
+                ),
+                format!(
+                    r#"{}"constellation":{}{:?}"#,
+                    indent,
+                    space,
+                    self.constellation().unwrap()
+                ),
+                format!(r#"{}"isValid":{}{:?}"#, indent, space, self.is_valid()),
+            ]
+        } else {
+            vec![
+                format!(r#"{}"number":{}{:?}"#, indent, space, self.number()),
+                format!(r#"{}"isValid":{}{:?}"#, indent, space, self.is_valid()),
+            ]
+        };
+
+        if pretty {
+            let s = props.join(",\n");
+            format!("{{\n{}\n}}", s)
+        } else {
+            let s = props.join(",");
+            format!("{{{}}}", s)
+        }
+    }
 }
 
 /// Returns the Chinese Zodiac animal by the given year, the given year
 /// should not be less than 1000.
-pub fn chinese_zodiac(year: u32) -> Option<String> {
+pub fn chinese_zodiac(year: u32) -> Option<&'static str> {
     if year < 1000 {
         return None;
     }
     let end = 3;
     let idx = (year - end) % 12;
     let zod = CHINESE_ZODIAC[idx as usize];
-    Some(zod.to_owned())
+    Some(zod)
 }
 
 /// Returns the Chinese Era by the given year, the given year
@@ -354,7 +480,7 @@ pub fn chinese_era(year: u32) -> Option<String> {
 }
 
 /// Returns the constellation by the given month and day.
-pub fn constellation(month: u32, day: u32) -> Option<String> {
+pub fn constellation(month: u32, day: u32) -> Option<&'static str> {
     let result = if (month == 1 && day >= 20) || (month == 2 && day <= 18) {
         "水瓶座"
     } else if (month == 2 && day >= 19) || (month == 3 && day <= 20) {
@@ -382,7 +508,7 @@ pub fn constellation(month: u32, day: u32) -> Option<String> {
     } else {
         return None;
     };
-    Some(result.to_owned())
+    Some(result)
 }
 
 /// Upgrades a Chinese ID number from 15-digit to 18-digit.
@@ -410,13 +536,13 @@ pub fn upgrade(number: &str) -> Result<String, Error> {
 
         let weight = get_weights_sum(&iarr);
         if let Some(code) = get_check_code(weight) {
-            idv2.push_str(&code);
+            idv2.push_str(code);
             Ok(idv2)
         } else {
             return Err(Error::UpgradeError);
         }
     } else {
-        Err(Error::UpgradeError)
+        Err(Error::InvalidNumber)
     }
 }
 
@@ -475,187 +601,6 @@ fn validate_v2(number: &str) -> bool {
     false
 }
 
-/// Generates a new fake ID number.
-pub fn new_fake(
-    region: &str,
-    year: i32,
-    month: i32,
-    date: i32,
-    gender: Gender,
-) -> Result<String, Error> {
-    if region.len() != 6 {
-        return Err(Error::GenerateFakeIDError(
-            "The length of region code must be 6 digits".to_string(),
-        ));
-    }
-    let mut rng = thread_rng();
-    let mut seq = rng.gen_range(0..999);
-
-    if gender == Gender::Male && seq % 2 == 0 {
-        seq += 1;
-    }
-    if gender == Gender::Female && seq % 2 == 1 {
-        seq += 1;
-    }
-
-    let birthdate_str = format!("{}{:0>2}{:0>2}", year, month, date);
-    let birth_date = NaiveDate::parse_from_str(&birthdate_str, "%Y%m%d");
-    if birth_date.is_err() {
-        return Err(Error::GenerateFakeIDError(
-            "Invalid birth of date".to_string(),
-        ));
-    }
-
-    let seg17 = format!("{}{}{:0>3}", region, birthdate_str, seq);
-
-    let iarr = match string_to_integer_array(&seg17) {
-        Ok(value) => value,
-        _ => return Err(Error::GenerateFakeIDError("Invalid characters".to_string())),
-    };
-
-    let weight = get_weights_sum(&iarr);
-    if let Some(code) = get_check_code(weight) {
-        Ok(seg17 + &code)
-    } else {
-        return Err(Error::GenerateFakeIDError("Invalid check code".to_string()));
-    }
-}
-
-/// Options which can be used to configure how a fake ID number is generated.
-#[derive(Debug, PartialEq, PartialOrd)]
-pub struct FakeOptions {
-    region: Option<String>,
-    min_year: Option<i32>,
-    max_year: Option<i32>,
-    gender: Option<Gender>,
-}
-
-impl FakeOptions {
-    /// Returns default options.
-    pub fn default() -> Self {
-        FakeOptions {
-            region: None,
-            min_year: None,
-            max_year: None,
-            gender: None,
-        }
-    }
-
-    /// Sets the minimum year(min_year <= max_year <= current).
-    pub fn set_min_year(&mut self, year: i32) {
-        self.min_year = Some(year);
-    }
-
-    /// Sets the maximum year(min_year <= max_year <= current).
-    pub fn set_max_year(&mut self, year: i32) {
-        self.max_year = Some(year);
-    }
-
-    /// Sets the region code, the length must be 2..6.
-    pub fn set_region(&mut self, code: &str) {
-        self.region = Some(code.to_owned());
-    }
-
-    /// Sets the gender.
-    pub fn set_gender(&mut self, gender: Gender) {
-        self.gender = Some(gender);
-    }
-}
-
-/// Generates a random fake ID number.
-pub fn rand_fake() -> Result<String, Error> {
-    let option = FakeOptions::default();
-    rand_fake_with_opts(&option)
-}
-
-/// Generates a random fake ID number using the given options.
-pub fn rand_fake_with_opts(opts: &FakeOptions) -> Result<String, Error> {
-    let region_code = if let Some(reg) = &opts.region {
-        match region::rand_code_starts_with(&reg) {
-            Some(code) => code,
-            _ => {
-                return Err(Error::GenerateFakeIDError(
-                    "Invalid region code".to_string(),
-                ))
-            }
-        }
-    } else {
-        region::rand_code()
-    };
-
-    let mut rng = thread_rng();
-    let now = Local::now();
-
-    if let Some(value) = opts.max_year {
-        if value > now.year() {
-            return Err(Error::GenerateFakeIDError(format!(
-                "Max year must be less than or equal to {}",
-                now.year()
-            )));
-        }
-    }
-
-    if let Some(value) = opts.min_year {
-        if value > now.year() {
-            return Err(Error::GenerateFakeIDError(format!(
-                "Min year must be less than or equal to {}",
-                now.year()
-            )));
-        }
-    }
-
-    if opts.min_year.is_some() && opts.max_year.is_some() {
-        let min = opts.min_year.unwrap();
-        let max = opts.max_year.unwrap();
-        if max < min {
-            return Err(Error::GenerateFakeIDError(
-                "Max year must be greater than or equal to min year".to_string(),
-            ));
-        }
-    }
-
-    let min_age = if let Some(y) = opts.max_year {
-        now.year() - y
-    } else {
-        0
-    };
-    let max_age = if let Some(y) = opts.min_year {
-        now.year() - y
-    } else {
-        100
-    };
-
-    let age = if max_age == min_age {
-        max_age
-    } else {
-        rng.gen_range(min_age..=max_age)
-    };
-
-    let days = rng.gen_range(1..365);
-    let dt = Local.ymd(now.year(), 1, 1);
-    let birth = dt - Duration::days((age * 365 - days) as i64);
-    let gender = if let Some(value) = &opts.gender {
-        match value {
-            Gender::Male => Gender::Male,
-            Gender::Female => Gender::Female,
-        }
-    } else {
-        let flag = rng.gen_range(0..10);
-        if flag % 2 == 0 {
-            Gender::Male
-        } else {
-            Gender::Female
-        }
-    };
-    new_fake(
-        &region_code,
-        birth.year(),
-        birth.month() as i32,
-        birth.day() as i32,
-        gender,
-    )
-}
-
 fn is_digital(s: &str) -> bool {
     if s.is_empty() {
         false
@@ -664,7 +609,7 @@ fn is_digital(s: &str) -> bool {
     }
 }
 
-fn get_check_code(sum: u32) -> Option<String> {
+fn get_check_code(sum: u32) -> Option<&'static str> {
     let code = match sum % 11 {
         10 => "2",
         9 => "3",
@@ -679,7 +624,7 @@ fn get_check_code(sum: u32) -> Option<String> {
         0 => "1",
         _ => return None,
     };
-    Some(code.to_string())
+    Some(code)
 }
 
 fn string_to_integer_array(s: &str) -> Result<Vec<u32>, Error> {
@@ -716,7 +661,7 @@ mod tests {
     fn upgrade_v1_to_v2() {
         let id = Identity::new("632123820927051");
         assert_eq!(id.is_valid(), true);
-        assert_eq!(id.number(), "632123198209270518".to_string());
+        assert_eq!(id.number(), "632123198209270518");
 
         let id = upgrade("310112850409522").unwrap();
         assert_eq!(&id, "310112198504095227");
@@ -731,7 +676,10 @@ mod tests {
     #[test]
     fn show_details() {
         let id = Identity::new("511702800222130");
-        print_details(&id);
+        println!("{}", id.to_json_string(true));
+
+        let id = Identity::new("51170280022213X");
+        println!("{}", id.to_json_string(false));
     }
 
     #[test]
@@ -745,23 +693,23 @@ mod tests {
 
     #[test]
     fn test_some_utilities() {
-        assert_eq!(chinese_zodiac(1000), Some("鼠".to_string()));
-        assert_eq!(chinese_zodiac(1900), Some("鼠".to_string()));
-        assert_eq!(chinese_zodiac(2021), Some("牛".to_string()));
+        assert_eq!(chinese_zodiac(1000), Some("鼠"));
+        assert_eq!(chinese_zodiac(1900), Some("鼠"));
+        assert_eq!(chinese_zodiac(2021), Some("牛"));
 
         assert_eq!(chinese_era(1000), Some("庚子".to_string()));
         assert_eq!(chinese_era(1900), Some("庚子".to_string()));
         assert_eq!(chinese_era(2021), Some("辛丑".to_string()));
 
-        assert_eq!(constellation(10, 25), Some("天蝎座".to_string()));
-        assert_eq!(constellation(2, 29), Some("双鱼座".to_string()));
+        assert_eq!(constellation(10, 25), Some("天蝎座"));
+        assert_eq!(constellation(2, 29), Some("双鱼座"));
         assert_eq!(constellation(0, 32), None);
     }
 
     #[test]
     fn query_region() {
         let name = region::query("511702").unwrap();
-        assert_eq!(&name, "四川省达州市通川区");
+        assert_eq!(name, "四川省达州市通川区");
     }
 
     #[test]
@@ -778,70 +726,4 @@ mod tests {
         let b = Identity::new("130133197909136078");
         assert_eq!(a != b, true);
     }
-
-    #[test]
-    fn generate_fake_id() {
-        let f = new_fake("654325", 2018, 2, 28, Gender::Male).unwrap();
-        let id = Identity::new(&f);
-        print_details(&id);
-        assert_eq!(id.is_valid(), true);
-
-        let f = new_fake("310104", 2020, 2, 29, Gender::Female).unwrap();
-        let id = Identity::new(&f);
-        print_details(&id);
-        assert_eq!(id.is_valid(), true);
-
-        for i in 1..=10 {
-            let num = rand_fake().unwrap();
-            println!("{}: {}", i, num);
-        }
-
-        let f = new_fake("2300", 1970, 2, 28, Gender::Male);
-        assert_eq!(f.is_err(), true);
-
-        let f = new_fake("230000", 1970, 2, 29, Gender::Male);
-        assert_eq!(f.is_err(), true);
-    }
-
-    #[test]
-    fn generate_fake_id_with_options() {
-        let mut opts = FakeOptions::default();
-        opts.set_region("3301");
-        opts.set_gender(Gender::Female);
-        opts.set_min_year(1990);
-        opts.set_max_year(2000);
-
-        for i in 1..=5 {
-            let num = rand_fake_with_opts(&opts).unwrap();
-            println!("{}: {:}", i, num);
-        }
-
-        opts.set_region("11");
-        opts.set_gender(Gender::Male);
-        for i in 1..=5 {
-            let num = rand_fake_with_opts(&opts).unwrap();
-            println!("{}: {:}", i, num);
-        }
-    }
-
-    fn print_details(id: &Identity) {
-        let detail = vec![
-            format!("Number: {:?}", id.number()),
-            format!("Age: {:?}", id.age()),
-            format!("Year: {:?}", id.year()),
-            format!("Month: {:?}", id.month()),
-            format!("Date: {:?}", id.date()),
-            format!("BirthDate: {:?}", id.birth_date()),
-            format!("ChineseEra: {:?}", id.chinese_era()),
-            format!("ChineseZodiac: {:?}", id.chinese_zodiac()),
-            format!("Constellation: {:?}", id.constellation()),
-            format!("Gender: {:?}", id.gender()),
-            format!("Province: {:?}", id.province()),
-            format!("IsValid: {:?}", id.is_valid()),
-            format!("IsEmpty: {:?}", id.is_empty()),
-            format!("NumberLength: {:?}", id.len()),
-            format!("Region: {:?}", id.region()),
-        ];
-        println!("#--------- Summary ---------#\n{}\n", detail.join("\n"));
-    }
 }
